--[[-------------------------------------------------------------
MACUILXOCHITL                                                   -
martblek (martblek@gmail.com)                                   -
...                                                             -
creates and deforms various waves which,                        -
with the help of old musical gods, are shaped into              -
the greatest possible harmony with the world                    -
-----------------------------------------------------------------


v0.011
Splitting the source code into parts
GUI changes
Removed welcome screen :)
Sample length is limited to specified values
Added waveforms other than sine wave with view to better repetition
Redesigned saving program settings
Midi mappings and key shortcut added
Some optimizations


v0.01
First public release



]]---------------------------------------------------------------
VERSION = "0.011"
AUTHOR = "martblek (martblek@gmail.com)"
DEBUG = false

PATCHES = nil
RECALCULATE = true   --< this is for hardcalc in redraw routine
MATH_BUFFER = {}     --< this stores calculated values
MIN_VALUE = 9999
MAX_VALUE = -9999
MAIN_SAMPLE = nil
MAIN_INSTRUMENT = nil

--[[------------------------------------------------------------- 
Register menu in renoise                                        -
]]---------------------------------------------------------------

require "src/files"
require "src/gui"
require "src/generators"
require "src/render"


renoise.tool():add_menu_entry{
  name = "Sample Editor: Macuilxochitl",
  invoke = function()
    show_gui()
  end
}

renoise.tool():add_keybinding{
  name="Global:Tools:Maciulxochitl",
  invoke=function()
    show_gui()
  end
}

-- Simple map function from one range to another range

local function map_from_to(val, fmin, fmax, tmin, tmax)
  return (val - fmin) / (fmax - fmin) * (tmax - tmin) + tmin
end

-------------------- OSC1 MIDI --------------------------------

if not renoise.tool():has_midi_mapping("MX:OSC1_selector") then
  renoise.tool():add_midi_mapping{
    name="MX:OSC1_selector",
    invoke=function(msg)
      local val = map_from_to(msg.int_value, 0, 127, 1, #WAVES)
      vb.views.osc1type.value = val
      tmp.SAMPLE_TYPE_1 = val
      redraw_sample()
    end
  }
end

if not renoise.tool():has_midi_mapping("MX:OSC1_period") then
  renoise.tool():add_midi_mapping{
    name="MX:OSC1_period",
    invoke=function(msg)
      local val = map_from_to(msg.int_value, 0, 127, 1, 255)
      vb.views.freq1slider.value = val
      tmp.SAMPLE_FREQUENCY_1 = val
    end
  }
end

if not renoise.tool():has_midi_mapping("MX:OSC1_amplitude") then
  renoise.tool():add_midi_mapping{
    name="MX:OSC1_amplitude",
    invoke=function(msg)
      local val = map_from_to(msg.int_value, 0, 127, -1, 1)
      vb.views.amp1slider.value = val
      tmp.SAMPLE_AMPLITUDE_1 = val
    end
  }
end

if not renoise.tool():has_midi_mapping("MX:OSC1_phase") then
  renoise.tool():add_midi_mapping{
    name="MX:OSC1_phase",
    invoke=function(msg)
      local val = map_from_to(msg.int_value, 0, 127, -def.SAMPLE_MAX_PHASE, def.SAMPLE_MAX_PHASE)
      vb.views.phase1slider.value = val
      tmp.SAMPLE_PHASE_1 = val
    end
  }
end

if not renoise.tool():has_midi_mapping("MX:OSC1_detail") then
  renoise.tool():add_midi_mapping{
    name="MX:OSC1_detail",
    invoke=function(msg)
      local val = map_from_to(msg.int_value, 0, 127, 1, 128)
      vb.views.detail1slider.value = val
      tmp.SAMPLE_DETAIL_1 = val
    end
  }
end

------------------------ OSC2 MIDI ----------------------------

if not renoise.tool():has_midi_mapping("MX:OSC2_selector") then
  renoise.tool():add_midi_mapping{
    name="MX:OSC2_selector",
    invoke=function(msg)
      local val = map_from_to(msg.int_value, 0, 127, 1, #WAVES)
      vb.views.osc2type.value = val
      tmp.SAMPLE_TYPE_2 = val
    end
  }
end

if not renoise.tool():has_midi_mapping("MX:OSC2_period") then
  renoise.tool():add_midi_mapping{
    name="MX:OSC2_period",
    invoke=function(msg)
      local val = map_from_to(msg.int_value, 0, 127, 1, 255)
      vb.views.freq2slider.value = val
      tmp.SAMPLE_FREQUENCY_2 = val
    end
  }
end

if not renoise.tool():has_midi_mapping("MX:OSC2_amplitude") then
  renoise.tool():add_midi_mapping{
    name="MX:OSC2_amplitude",
    invoke=function(msg)
      local val = map_from_to(msg.int_value, 0, 127, -1, 1)
      vb.views.amp2slider.value = val
      tmp.SAMPLE_AMPLITUDE_2 = val
    end
  }
end

if not renoise.tool():has_midi_mapping("MX:OSC2_phase") then
  renoise.tool():add_midi_mapping{
    name="MX:OSC2_phase",
    invoke=function(msg)
      local val = map_from_to(msg.int_value, 0, 127, -def.SAMPLE_MAX_PHASE, def.SAMPLE_MAX_PHASE)
      vb.views.phase2slider.value = val
      tmp.SAMPLE_PHASE_2 = val
    end
  }
end

if not renoise.tool():has_midi_mapping("MX:OSC2_detail") then
  renoise.tool():add_midi_mapping{
    name="MX:OSC2_detail",
    invoke=function(msg)
      local val = map_from_to(msg.int_value, 0, 127, 1, 128)
      vb.views.detail2slider.value = val
      tmp.SAMPLE_DETAIL_2 = val
    end
  }
end

------------------------- OSC3 MIDI -----------------------------

if not renoise.tool():has_midi_mapping("MX:OSC3_selector") then
  renoise.tool():add_midi_mapping{
    name="MX:OSC3_selector",
    invoke=function(msg)
      local val = map_from_to(msg.int_value, 0, 127, 1, #WAVES)
      vb.views.osc3type.value = val
      tmp.SAMPLE_TYPE_3 = val
    end
  }
end

if not renoise.tool():has_midi_mapping("MX:OSC3_period") then
  renoise.tool():add_midi_mapping{
    name="MX:OSC3_period",
    invoke=function(msg)
      local val = map_from_to(msg.int_value, 0, 127, 1, 255)
      vb.views.freq3slider.value = val
      tmp.SAMPLE_FREQUENCY_3 = val
    end
  }
end

if not renoise.tool():has_midi_mapping("MX:OSC3_amplitude") then
  renoise.tool():add_midi_mapping{
    name="MX:OSC3_amplitude",
    invoke=function(msg)
      local val = map_from_to(msg.int_value, 0, 127, -1, 1)
      vb.views.amp3slider.value = val
      tmp.SAMPLE_AMPLITUDE_3 = val
    end
  }
end

if not renoise.tool():has_midi_mapping("MX:OSC3_phase") then
  renoise.tool():add_midi_mapping{
    name="MX:OSC3_phase",
    invoke=function(msg)
      local val = map_from_to(msg.int_value, 0, 127, -def.SAMPLE_MAX_PHASE, def.SAMPLE_MAX_PHASE)
      vb.views.phase3slider.value = val
      tmp.SAMPLE_PHASE_3 = val
    end
  }
end

if not renoise.tool():has_midi_mapping("MX:OSC3_detail") then
  renoise.tool():add_midi_mapping{
    name="MX:OSC3_detail",
    invoke=function(msg)
      local val = map_from_to(msg.int_value, 0, 127, 1, 128)
      vb.views.detail3slider.value = val
      tmp.SAMPLE_DETAIL_3 = val
    end
  }
end

------------------------- NOISE AMOUNT --------------------------

if not renoise.tool():has_midi_mapping("MX:Noise_amount") then
  renoise.tool():add_midi_mapping{
    name="MX:Noise_amount",
    invoke=function(msg)
      local val = map_from_to(msg.int_value, 0, 127, 0, def.SAMPLE_MAX_NOISE)
      vb.views.noise.value = val
      tmp.SAMPLE_NOISE_AMOUNT = val
      if val < 1 then
        tmp.SAMPLE_NOISE_BOOL = false
        vb.views.noisy_message.text="Sak Nik sleeps"
      else
        tmp.SAMPLE_NOISE_BOOL = true
        vb.views.noisy_message.text="Sak Nik is awake"
      end
    end
  }
end

------------------------- SAMPLE LEN ----------------------------

if not renoise.tool():has_midi_mapping("MX:SAMPLE_LEN_selector") then
  renoise.tool():add_midi_mapping{
    name="MX:SAMPLE_LEN_selector",
    invoke=function(msg)
      local val = map_from_to(msg.int_value, 0, 127, 1, #BUFFERS)
      vb.views.framepopup.value = val
      tmp.SAMPLE_POPUP = val
      tmp.SAMPLE_FRAMES = FRAMES[tmp.SAMPLE_POPUP]
    end
  }
end

----------------------------- OPERATIONS -------------------------------

if not renoise.tool():has_midi_mapping("MX:OP1_selector") then
  renoise.tool():add_midi_mapping{
    name="MX:OP1_selector",
    invoke=function(msg)
      local val = map_from_to(msg.int_value, 0, 127, 1, #OPERATIONS)
      vb.views.op1.value = val
      tmp.OP1 = val
    end
  }
end

if not renoise.tool():has_midi_mapping("MX:OP2_selector") then
  renoise.tool():add_midi_mapping{
    name="MX:OP2_selector",
    invoke=function(msg)
      local val = map_from_to(msg.int_value, 0, 127, 1, #OPERATIONS)
      vb.views.op2.value = val
      tmp.OP2 = val
    end
  }
end

---------------------------- GAUSSIANS ------------------------------

if not renoise.tool():has_midi_mapping("MX:Gaussian_toggle") then
  renoise.tool():add_midi_mapping{
    name="MX:Gaussian_toggle",
    invoke=function(msg)
      local val = map_from_to(msg.int_value, 0, 127, 0, 1)
      if val == 1 then
        vb.views.gaussian_on_off.value = true
        tmp.GAUSSIAN_ON_OFF = true
      else
        vb.views.gaussian_on_off.value = false
        tmp.GAUSSIAN_ON_OFF = false
      end
    end
  }
end

if not renoise.tool():has_midi_mapping("MX:G1_rising") then
  renoise.tool():add_midi_mapping{
    name="MX:G1_rising",
    invoke=function(msg)
      local val = map_from_to(msg.int_value, 0, 127, def.GAUSSIAN_V_MIN, def.GAUSSIAN_V_MAX)
      vb.views.gauss_v1.value = val
      tmp.GAUSSIAN_V1 = val
    end
  }
end

if not renoise.tool():has_midi_mapping("MX:G2_rising") then
  renoise.tool():add_midi_mapping{
    name="MX:G2_rising",
    invoke=function(msg)
      local val = map_from_to(msg.int_value, 0, 127, def.GAUSSIAN_V_MIN, def.GAUSSIAN_V_MAX)
      vb.views.gauss_v2.value = val
      tmp.GAUSSIAN_V2 = val
    end
  }
end

if not renoise.tool():has_midi_mapping("MX:G3_rising") then
  renoise.tool():add_midi_mapping{
    name="MX:G3_rising",
    invoke=function(msg)
      local val = map_from_to(msg.int_value, 0, 127, def.GAUSSIAN_V_MIN, def.GAUSSIAN_V_MAX)
      vb.views.gauss_v3.value = val
      tmp.GAUSSIAN_V3 = val
    end
  }
end

if not renoise.tool():has_midi_mapping("MX:G4_rising") then
  renoise.tool():add_midi_mapping{
    name="MX:G4_rising",
    invoke=function(msg)
      local val = map_from_to(msg.int_value, 0, 127, def.GAUSSIAN_V_MIN, def.GAUSSIAN_V_MAX)
      vb.views.gauss_v4.value = val
      tmp.GAUSSIAN_V4 = val
    end
  }
end

if not renoise.tool():has_midi_mapping("MX:G5_rising") then
  renoise.tool():add_midi_mapping{
    name="MX:G5_rising",
    invoke=function(msg)
      local val = map_from_to(msg.int_value, 0, 127, def.GAUSSIAN_V_MIN, def.GAUSSIAN_V_MAX)
      vb.views.gauss_v5.value = val
      tmp.GAUSSIAN_V5 = val
    end
  }
end

if not renoise.tool():has_midi_mapping("MX:G1_dancing") then
  renoise.tool():add_midi_mapping{
    name="MX:G1_dancing",
    invoke=function(msg)
      local val = map_from_to(msg.int_value, 0, 127, def.GAUSSIAN_N_MIN, def.GAUSSIAN_N_MAX)
      vb.views.gauss_n1.value = val
      tmp.GAUSSIAN_N1 = val
    end
  }
end

if not renoise.tool():has_midi_mapping("MX:G2_dancing") then
  renoise.tool():add_midi_mapping{
    name="MX:G2_dancing",
    invoke=function(msg)
      local val = map_from_to(msg.int_value, 0, 127, def.GAUSSIAN_N_MIN, def.GAUSSIAN_N_MAX)
      vb.views.gauss_n2.value = val
      tmp.GAUSSIAN_N2 = val
    end
  }
end

if not renoise.tool():has_midi_mapping("MX:G3_dancing") then
  renoise.tool():add_midi_mapping{
    name="MX:G3_dancing",
    invoke=function(msg)
      local val = map_from_to(msg.int_value, 0, 127, def.GAUSSIAN_N_MIN, def.GAUSSIAN_N_MAX)
      vb.views.gauss_n3.value = val
      tmp.GAUSSIAN_N3 = val
    end
  }
end

if not renoise.tool():has_midi_mapping("MX:G4_dancing") then
  renoise.tool():add_midi_mapping{
    name="MX:G4_dancing",
    invoke=function(msg)
      local val = map_from_to(msg.int_value, 0, 127, def.GAUSSIAN_N_MIN, def.GAUSSIAN_N_MAX)
      vb.views.gauss_n4.value = val
      tmp.GAUSSIAN_N4 = val
    end
  }
end

if not renoise.tool():has_midi_mapping("MX:G5_dancing") then
  renoise.tool():add_midi_mapping{
    name="MX:G5_dancing",
    invoke=function(msg)
      local val = map_from_to(msg.int_value, 0, 127, def.GAUSSIAN_N_MIN, def.GAUSSIAN_N_MAX)
      vb.views.gauss_n5.value = val
      tmp.GAUSSIAN_N5 = val
    end
  }
end

if not renoise.tool():has_midi_mapping("MX:G1_expands") then
  renoise.tool():add_midi_mapping{
    name="MX:G1_expands",
    invoke=function(msg)
      local val = map_from_to(msg.int_value, 0, 127, def.GAUSSIAN_S_MIN, def.GAUSSIAN_S_MAX)
      vb.views.gauss_s1.value = val
      tmp.GAUSSIAN_S1 = val
    end
  }
end

if not renoise.tool():has_midi_mapping("MX:G2_expands") then
  renoise.tool():add_midi_mapping{
    name="MX:G2_expands",
    invoke=function(msg)
      local val = map_from_to(msg.int_value, 0, 127, def.GAUSSIAN_S_MIN, def.GAUSSIAN_S_MAX)
      vb.views.gauss_s2.value = val
      tmp.GAUSSIAN_S2 = val
    end
  }
end

if not renoise.tool():has_midi_mapping("MX:G3_expands") then
  renoise.tool():add_midi_mapping{
    name="MX:G3_expands",
    invoke=function(msg)
      local val = map_from_to(msg.int_value, 0, 127, def.GAUSSIAN_S_MIN, def.GAUSSIAN_S_MAX)
      vb.views.gauss_s3.value = val
      tmp.GAUSSIAN_S3 = val
    end
  }
end

if not renoise.tool():has_midi_mapping("MX:G4_expands") then
  renoise.tool():add_midi_mapping{
    name="MX:G4_expands",
    invoke=function(msg)
      local val = map_from_to(msg.int_value, 0, 127, def.GAUSSIAN_S_MIN, def.GAUSSIAN_S_MAX)
      vb.views.gauss_s4.value = val
      tmp.GAUSSIAN_S4 = val
    end
  }
end

if not renoise.tool():has_midi_mapping("MX:G5_expands") then
  renoise.tool():add_midi_mapping{
    name="MX:G5_expands",
    invoke=function(msg)
      local val = map_from_to(msg.int_value, 0, 127, def.GAUSSIAN_S_MIN, def.GAUSSIAN_S_MAX)
      vb.views.gauss_s5.value = val
      tmp.GAUSSIAN_S5 = val
    end
  }
end

-------------------------------- this is the END --------------------------------------------