--[[---------------------------------------------------------------------
Wave Generators                                                         -
]]-----------------------------------------------------------------------

Oscilator = {
  class=1,
  frequency = 2,
  phase=0,
  amplitude=0.5,
  detail=1,
}

function Oscilator:new(osc,freq, amp, phase, detail, class)
  osc = osc or {}
  setmetatable(osc, self)
  self.__index = self
  self.freq = freq or 1
  self.amplitude = amp or 0.5
  self.phase = phase or 1
  self.detail = detail or 1
  self.class = class or 1
  return osc
end

OSC1 = Oscilator:new(nil, tmp.SAMPLE_FREQUENCY_1, tmp.SAMPLE_AMPLITUDE_1, tmp.SAMPLE_PHASE_1, tmp.SAMPLE_DETAIL_1, tmp.SAMPLE_TYPE_1)
OSC2 = Oscilator:new(nil, tmp.SAMPLE_FREQUENCY_2, tmp.SAMPLE_AMPLITUDE_2, tmp.SAMPLE_PHASE_2, tmp.SAMPLE_DETAIL_2, tmp.SAMPLE_TYPE_2)
OSC3 = Oscilator:new(nil, tmp.SAMPLE_FREQUENCY_3, tmp.SAMPLE_AMPLITUDE_3, tmp.SAMPLE_PHASE_3, tmp.SAMPLE_DETAIL_3, tmp.SAMPLE_TYPE_3)

function Oscilator:simple_sine(frame, sample_len)
	local pi = math.pi
	local sin = math.sin
	return self.amplitude * sin(2 * pi * self.frequency * ((frame + self.phase) / sample_len))
end


function Oscilator:simple_triangle(frame, sample_len)
	local pi = math.pi
	local asin = math.asin
	local sin = math.sin
	local floor = math.floor
	local twopi = 2 * pi
	local freq = floor(sample_len / self.frequency)
	return (2 * self.amplitude / pi) * asin(sin((twopi / freq) * (frame + self.phase)))
end


function Oscilator:simple_saw(frame, sample_len)
  local pi = math.pi
  local tan = math.tan
  local atan = math.atan
  local floor = math.floor
  local temp = (pi*(frame + self.phase)) / floor(sample_len / self.frequency)
  return (-(2 * self.amplitude) / pi) * atan(1 / tan(temp))
end


function Oscilator:simple_square(frame, sample_len)
	local pi = math.pi
	local sin = math.sin
	local function sgn(x)
  		return x > 0 and 1 or (x == 0 and 0 or -1)
	end
	return self.amplitude * sgn(sin(2 * pi * (frame + self.phase) / (sample_len / self.frequency)))
end


function Oscilator:additive_saw(frame, sample_len)
  local sin = math.sin
  local pi = math.pi
  local abs = math.abs
  local floor = math.floor
  local tail = 8 / pi^2
  local test = floor(sample_len / self.frequency)

  local g = 0
  for n=1, abs(floor(self.detail)) do
    g = g + (1 ^(n-1)) / n * sin((2 * pi * n * (frame + self.phase)) / test)
  end

  local h = self.amplitude * tail * g

  return h
end


function Oscilator:additive_square(frame, sample_len)
  local sin = math.sin
  local pi = math.pi
  local abs = math.abs
  local floor = math.floor
  local tail = 4 / pi
  local test = floor(sample_len / self.frequency)

  local g = 0
  for n=1, abs(floor(self.detail)) do
    local part_1=(2*n-1)
    local part_2=(part_1* 2 * pi * (frame + self.phase))
    local part_3=(1/part_1)*sin(part_2 / test)
    g = g + part_3 
  end

  local h = self.amplitude * tail * g

  return h
end
